#include <stdbool.h>
#include <stdint.h>
#include <Energia.h>
#include "Wire.h"
#include "Melody_ADXL345.h"


//------------------Constructors---------------------------//
CMelodyADXL345::CMelodyADXL345(void)
{
    m_dataReady = false;
}

//-----------Private Methods------------------------------//
/********************************************************************************
 * @brief:  写单字节数据
 * @param：  regAddress,寄存器地址
 *           data，待写入数据
 * @return： none
 ********************************************************************************/
void CMelodyADXL345::write(uint8_t regAddress,uint8_t data)
{
    write(regAddress,&data,1);
}

/*********************************************************************************
 * @brief:  写入多个字节数据
 * @param： regAddress,寄存器地址
 *          pdata，指向待写入数据的指针
 *          length，待写入数据长度
 * @return：none
 *********************************************************************************/
void CMelodyADXL345::write(uint8_t regAddress,uint8_t *pdata,uint8_t length)
{
    Wire.beginTransmission(ADXLADDRESS);
    Wire.write(regAddress);
    Wire.write(pdata,length);
    Wire.endTransmission();
}

/**********************************************************************************
 * @brief:  读取单字节数据
 * @param：  regAddress,寄存器地址
 * @return: 读取的数据
 **********************************************************************************/
uint8_t CMelodyADXL345::read(uint8_t regAddress)
{
    uint8_t data = 0;
    Wire.beginTransmission(ADXLADDRESS);
    Wire.write(regAddress);
    Wire.endTransmission();

    Wire.requestFrom(ADXLADDRESS,1);
    if(Wire.available() >= 1)
    {
        data = Wire.read();
    }
    return data;
}

/************************************************************************************
 * @brief:  读取多字节数据
 * @param： regAddress,寄存器地址
 *          pdata，指向存放读取数据缓存指针
 *          length,待读取数据长度
 * @return：实际读取数据长度
 ************************************************************************************/
uint8_t CMelodyADXL345::read(uint8_t regAddress,uint8_t *pdata,uint8_t length)
{
    uint8_t readlength = 0;
    Wire.beginTransmission(ADXLADDRESS);
    Wire.write(regAddress);
    Wire.endTransmission();

    Wire.requestFrom(ADXLADDRESS,length);
    while(Wire.available())
    {
        pdata[readlength++] = Wire.read();
    }

    return readlength;
}

/*************************************************************************************
 * @brief:  初始化序列
 * @param： none
 * @return: none
 *************************************************************************************/
void CMelodyADXL345::startSequence(void)
{
    write(ADXLREG_DATAFORMAT,ADXLVALUE_DATAFORMAT);
    write(ADXLREG_BWRATE,ADXLVALUE_BWRATE);
    write(ADXLREG_POWERCTL,ADXLVALUE_POWERCTL);
    write(ADXLREG_INTENABLE,ADXLVALUE_INTENABLE);
    write(ADXLREG_OFSX,ADXLVALUE_OFSX);
    write(ADXLREG_OFSY,ADXLVALUE_OFSY);
    write(ADXLREG_OFSZ,ADXLVALUE_OFSZ);
}

//---------------Public Methods-------------------------//
/************************************************************************************
 * @brief:  初始化配置
 * @param：  none
 * @return： none
 ************************************************************************************/
void CMelodyADXL345::begin(void)
{
    //此次完成中断初始配置。库函数默认下没有实现初始过程。需要实现参考LDC1000模块的begin函数

    Wire.begin();

    startSequence();

    delayMicroseconds(200);
}

/***********************************************************************************
 * @brief:  Data Ready中断复位函数
 * @param：  none
 * @return：none
 ***********************************************************************************/
void CMelodyADXL345::dataReadyHandler(void)
{
    m_dataReady = true;
}

/**********************************************************************************
 * @brief:  Data Ready中断有效信号
 * @param： none
 * @return：none
 **********************************************************************************/
bool CMelodyADXL345::available(void)
{
    if(m_dataReady)
    {
        m_dataReady = false;
        return true;
    }
    else
    {
        return false;
    }
}

/**********************************************************************************
 * @brief:  获取X轴数据
 * @param： none
 * @return：X轴数据
 **********************************************************************************/
int16_t CMelodyADXL345::getXAccelerate(void)
{
    uint8_t data[2] = {0};
    read(ADXLREG_DATAX0,data,2);
    
    return ((uint16_t)(data[1] << 8) | data[0]);
}

/**********************************************************************************
 * @brief:  获取Y轴数据
 * @param： none
 * @return：Y轴数据
 **********************************************************************************/
int16_t CMelodyADXL345::getYAccelerate(void)
{
    uint8_t data[2] = {0};
    read(ADXLREG_DATAY0,data,2);
    
    return ((uint16_t)(data[1] << 8) | data[0]);
}

/**********************************************************************************
 * @brief:  获取Z轴数据
 * @param： none     
 * @return：Z轴数据
 **********************************************************************************/
int16_t CMelodyADXL345::getZAccelerate(void)
{
    uint8_t data[2] = {0};
    read(ADXLREG_DATAZ0,data,2);
    
    return ((uint16_t)(data[1] << 8) | data[0]);
}

/**********************************************************************************
 * @brief:  获取三轴数据
 * @param：  none
 * @return：三轴数据
 **********************************************************************************/
t_Accelerate CMelodyADXL345::getAccelerate(void)
{
    uint8_t data[6] = {0};
    read(ADXLREG_DATAZ0,data,6);

    t_Accelerate accelerate = {0};
    accelerate.X = (uint16_t)(data[1] << 8) | data[0];
    accelerate.Y = (uint16_t)(data[3] << 8) | data[2];
    accelerate.Z = (uint16_t)(data[5] << 8) | data[4];

    return accelerate;
}

/**********************************************************************************
 * @brief:  获取设备ID
 * @param：  none
 * @return：设备ID
 **********************************************************************************/
uint8_t CMelodyADXL345::getDeviceID(void)
{
    return read(ADXLREG_DEVICEID);
}


void interruptHandler(void)
{
    MelodyADXL345.dataReadyHandler();
}

CMelodyADXL345 MelodyADXL345;