#include <stdint.h>
#include <stdbool.h>
#include "Melody_BP.h"


static const uint16_t startpoint = 180;         //单位mmHg
static const uint16_t stoppoint = 20;           //单位mmHg
static const float resolution = 0.25;           //单位mV/mmHg(MPS3117,5.8PSI=300mmHg,75mV，75/300=0.25)   
static const float gain = 28.7;                 //模块放大倍数
    

//---------private method---------------

/******************************************************
 * @brief: 将电压值转换为压力值。硬件设计上将0~180mmHg(0~45mV) 
 *         最近通过放大电路接入MCU，放大倍数为gian（28.7）       
 * @param: sample,电压采样值
 * @return：mmHg
 *****************************************************/
float CMelodyBP::transfer(uint16_t sample)
{
    float pressure = 1.0 * map(sample,0,4095,0,1460) / gain / resolution;   
     
    return pressure;
}

/********************************************************
 * @brief:  获取静压力（袖套压力）
 * @param： none
 * @return: 获取的压力值
 ********************************************************/
float CMelodyBP::readStaticBP(void)
{
    return (transfer(analogRead(sbp_pin)));
}

/********************************************************
 * @brief: 获取波动压力
 * @param: none
 * @return:获取的压力值
 ********************************************************/
float CMelodyBP::readWaveBP(void)
{
    return (transfer(analogRead(wbp_pin)));
}  


/*********************************************************
 * @brief: 读取血压参数：袖套压+波动 AD采样
 * @param： none
 * @return：返回数据长度
 **********************************************************/
uint16_t CMelodyBP::readBloodPressure(void)
{
    if(m_buffertindex >= bufferLength)
    {
        memmove(m_SBPbuffer,m_SBPbuffer + 1,sizeof(float) * (bufferLength - 1));
        memmove(m_WBPbuffer,m_WBPbuffer + 1,sizeof(float) * (bufferLength - 1));
        --m_buffertindex;
    }
    m_SBPbuffer[m_buffertindex] = readStaticBP();
    m_WBPbuffer[m_buffertindex] = readWaveBP();
    
    ++m_buffertindex;
    delay(10);
    return m_buffertindex;
}   



//--------public method------------------
/******************************************************
 * @brief: 构造函数
 *******************************************************/
CMelodyBP::CMelodyBP(void)
{
    memset(m_SBPbuffer,0,sizeof(float) * bufferLength);
    memset(m_WBPbuffer,0,sizeof(float) * bufferLength);
    m_buffertindex = 0;
    m_start = false;
    m_rxComplete = false;
}

/********************************************************
 * @brief:  初始配置
 * @param： none
 * @return：none
 ********************************************************/
void CMelodyBP::begin(void)
{
   pinMode(RED_LED ,OUTPUT);
}


/***********************************************************
 * @brief: 测试过程
 * @param：none
 * @return：测试是否完成
 * @description：当袖套压大于startpoint时开始采集波动，并存储数据
 *               当袖套压小于stoppoint时停止采集，发送数据至PC端
 **********************************************************/
bool CMelodyBP::measurement(void)
{
    //上升过程，判断数据缓存起始点
    if(readStaticBP() > startpoint)
    {
        delay(200);
        if(readStaticBP() > startpoint)
        {
            m_start = true;
            digitalWrite(RED_LED, HIGH);
        }
    }
    if(m_start)
    {

        readBloodPressure();

        if(m_SBPbuffer[m_buffertindex - 1] < stoppoint)        //判断数据缓存结束点 
        {
            m_start = false;
            digitalWrite(RED_LED, LOW);
            return true;
        }
        
    }
    return false;
}

/*******************************************************
 * @brief:  发送数据至PC端，包括静态压数据和波动数据
 * @param：h_serial,串口引用
 * @return：none
 * @description：首先发送袖套静态压数据（配合PC端，格式为DS+多个数据+\n，数据之间使用空格分开）
 *               发送完成后，发送波动数据（配合PC端，格式为DW+多个数据+\n，数据之间使用空格分开）
 *               数据格式为ASCII
 *******************************************************/
void CMelodyBP::transmitBoolPressure(HardwareSerial &h_serial)
{
    uint16_t index = 0;
    //静态压数据格式
    h_serial.print("DS");
    for(;index < m_buffertindex;++index)
    {
        Serial.print(m_SBPbuffer[index]);
        Serial.print(" ");
    }
    h_serial.print("\n");

    //波动数据格式
    h_serial.print("DW");
    for(index = 0;index < m_buffertindex;++index)
    {
        h_serial.print(m_WBPbuffer[index]);
        h_serial.print(" ");
    }
    h_serial.print("\n");

    clear();
}

/*************************************************************
 * @brief:  接收串口数据
 * @param： h_serial，串口引用
 *************************************************************/
char* CMelodyBP::receiveComPacket(HardwareSerial &h_serial,char* p_Combuffer,uint8_t Comlength)
{
    static uint8_t index = 0;
    while(h_serial.available())
    {
        char inChar = (char)h_serial.read();
        if(inChar != '\n')
        {
            p_Combuffer[index++] = inChar;
            index = (index >= Comlength) ? (Comlength - 1) : index;
        }
        else
        {
            p_Combuffer[index] = '\0';
            index = 0;      
            m_rxComplete = true;
        }
    }
}

/**************************************************************
 * @brief:  数据接收完成
 * @param：none
 * @return：true：接收完成   false: 接收未完成
 ***************************************************************/
bool CMelodyBP::available(void)
{
    return m_rxComplete;
}

/**********************************************************
 * @brief:  处理与上位机的通信
 * @param：  h_serial，串口引用
 *           p_Combuffer，接收的数据指针
 * @return:  返回数据包表示的运行状态信息
 ***********************************************************/        
eComState CMelodyBP::handleComPacket(HardwareSerial &h_serial,char* p_Combuffer)
{

    m_rxComplete = false;
    
    if(!strcmp(p_Combuffer,"R"))
    {
        h_serial.print("OK\n");
        return MEASURE;
    }

    if(!(strcmp(p_Combuffer,"L")))
    {
        h_serial.print("OK\n");
        return ONLINE;
    }

    if(!(strcmp(p_Combuffer,"T")))
    {
        h_serial.print("\n\n\n");
        return RESET;
    }
    return IDLE;
}

/********************************************************
 * @brief:  清除数据
 * @none：   none
 * @none：   none
 ********************************************************/
 void CMelodyBP::clear(void)
 {
    m_start = false;
    m_buffertindex = 0;                                     //发送完成缓存索引清零
    memset(m_SBPbuffer,0,sizeof(float) * bufferLength);     //数据清零
    memset(m_WBPbuffer,0,sizeof(float) * bufferLength);
    digitalWrite(RED_LED, LOW);
    m_rxComplete = false;
 }

CMelodyBP MelodyBP;