#include <stdbool.h>
#include <stdint.h>
#include <Energia.h>
#include "SPI.h"
#include "Melody_LDC1000.h"

static const float EPSINON = 0.00001;
/***********************************************************************************************
 * @brief:  写数据
 * @param:  regAddr,寄存器地址
 *          data,数据
 * @return：none
 ***********************************************************************************************/
void CMelodyLDC1000::write(uint8_t regAddr,uint8_t data)
{
    digitalWrite(LDCSPI_CS,LOW);               

    SPI.transfer(regAddr | LDCSPI_WBIT);
    SPI.transfer(data);

    digitalWrite(LDCSPI_CS,HIGH);
}

/***********************************************************************************************
 * @brief:  读取数据
 * @param： regAddr,寄存器地址
 * @return：none
 ***********************************************************************************************/
uint8_t CMelodyLDC1000::read(uint8_t regAddr)
{
    uint8_t result = 0;

    digitalWrite(LDCSPI_CS,LOW);

    SPI.transfer(regAddr | LDCSPI_RBIT);
    result = SPI.transfer(0x00);

    digitalWrite(LDCSPI_CS,HIGH);

    return result;
}

/************************************************************************************************
 * @brief:  初始化配置
 * @param： none
 * @return：none
 ************************************************************************************************/
void CMelodyLDC1000::begin()
{
    m_dataready = false;
    pinMode(LDCSPI_INT,INPUT_PULLDOWN);
    attachInterrupt(LDCSPI_INT,DataReadyIntHandle,FALLING);

    pinMode(LDCSPI_CS,OUTPUT);
    SPI.begin();

    //Start LDC1000
    write(LDCREG_RPMAX,LDCVALUE_RPMAX);
    write(LDCREG_RPMIN,LDCVALUE_RPMIN);
    write(LDCREG_SENSORFREQENCY,LDCVALUE_SENSORFREQENCY);
    write(LDCREG_LDCCONFIG,LDCVALUE_LDCCONFIG);
    write(LDCREG_CLKCONFIG,LDCVALUE_CLKCONFIG);

    write(LDCREG_THRESHOLDHIGH_LSB,LDCVALUE_THRESHOLDHIGH_LSB);
    write(LDCREG_THRESHOLDHIGH_MSB,LDCVALUE_THRESHOLDHIGH_MSB);
    write(LDCREG_THRESHOLDLOW_LSB,LDCVALUE_THRESHOLDLOW_LSB);
    write(LDCREG_THRESHOLDLOW_MSB,LDCVALUE_THRESHOLDLOW_MSB);
    write(LDCREG_PWRCONFIG,LDCVALUE_PWRCONFIG);

    write(LDCREG_INTCONFIG,LDCVALUE_INTCONFIG);
}

/**************************************************************************************
 * @brief:中断服务函数
 * @param：none
 * @return：none
 *************************************************************************************/
 void CMelodyLDC1000::interruptHandle(void)
 {
     m_dataready = true;
 }

/*************************************************************************************
 * @brief:  中断有效，可进行数据读取
 * @param： none
 * @return：true：有效 false：无效
 *************************************************************************************/
bool CMelodyLDC1000::available(void)
{
    if(m_dataready)
    {
        m_dataready = false;
        return true;
    }
    else
    {
        return false;
    }
}
/***************************************************************************************************
 * @brief:  读取Proximity data，两个字节长度
 * @param：  none
 * @return： Proximity data
 ***************************************************************************************************/
uint16_t CMelodyLDC1000::getProximity()
{
    uint16_t temp[2] = {0};
    uint16_t proximity = 0;

    temp[0] = read(LDCREG_PROXIMITY_LSB) & 0x00FF;
    temp[1] = read(LDCREG_PROXIMITY_MSB) & 0x00FF;

    proximity = (temp[1] << 8) | temp[0];

    return proximity;
}

/**************************************************************************************************
 * @brief:  读取Frequency Counter,三个字节长度
 * @param：none
 * @return: Frequency Counter data
 **************************************************************************************************/
uint32_t CMelodyLDC1000::getFrequency()
{
    uint32_t count[3] = {0};
    uint32_t frequency = 0;

    count[0] = read(LDCREG_FREQUENCYCTR_LSB) & 0x000000FF;
    count[1] = read(LDCREG_FREQUENCYCTR_MID) & 0x000000FF;
    count[2] = read(LDCREG_FREQUENCYCTR_MSB) & 0x000000FF;
 
    frequency = (count[2] << 16) | (count[1] << 8) | count[0]; 
    return frequency;
}

/**********************************************************************************************
 * @brief：计算电感值
 * @param：frequency，频率值，函数getFrequency的返回值
 * @return：电感值
 **********************************************************************************************/
float CMelodyLDC1000::calcInductance(uint32_t frequency)
{
    if(!frequency)
    {
        return 0.0;                 //frequency将会作为除数
    }

    float sensorFreq = (1.0 /3) * (1.0 * LDCFREQUENCY_EXT / frequency) * LDCRESPONSE_TIME;
    if((sensorFreq >= -EPSINON) && (sensorFreq <= EPSINON))
    {
        return 0.0;
    }
    float inductance = 1.0 * 1000000 / (LDCPARALLEI_CAP * pow(TWO_PI * sensorFreq,2));

    return inductance;
}

/***********************************************************************************************
 * @brief:  获取设备ID
 * @param： none
 * @return： device ID
 ***********************************************************************************************/
uint8_t CMelodyLDC1000::getDeviceID()
{
    return read(LDCREG_DEVICEID);
}

/*********************************************************************************************
 * @brief:  获取状态信息
 * @param:  none
 * @return: status
 *********************************************************************************************/
uint8_t CMelodyLDC1000::getStatus()
{
    return read(LDCREG_STATUS);
}

void DataReadyIntHandle(void)
{
    MelodyLDC1000.interruptHandle();
}
CMelodyLDC1000 MelodyLDC1000;