#include <stdbool.h>
#include <stdint.h>
#include <Energia.h>
#include "Wire.h"
#include "Melody_LiBattery.h"


//---------------private methods--------------------------//
/*************************************************************************
 * @brief:  写单字节数据
 * @param： cmd：command code
 *          data：待写入的数据
 * @return： none
 *************************************************************************/
void CMelodyLiBattery::write(uint8_t cmd,uint8_t data)
{
   write(cmd,&data,1);
}

/*************************************************************************
 * @brief:  写两个字节数据
 * @param： cmd：command code
 *          data：待写入的数据
 * @return：none
 *************************************************************************/
void CMelodyLiBattery::write(uint8_t cmd,uint16_t data)
{
    uint8_t buffer[2] = {0};
    buffer[0] = data & 0xFF;                            //LSB
    buffer[1] = (data >> 8) & 0xFF;
    write(cmd,buffer,2);
}

/*************************************************************************
 * @brief:  写入多个字节数据
 * @param： cmd：command code
 *          pdata:指向待写入数据的指针
 *          length：待写入数据长度
 * @return：none
 ************************************************************************/
void CMelodyLiBattery::write(uint8_t cmd,uint8_t *pdata,uint8_t length)
{
    Wire.beginTransmission(BQADDRESS);
    Wire.write(cmd);
    Wire.write(pdata,length);
    Wire.endTransmission();
    delay(1);
}

/*************************************************************************
 * @brief:  读取两个字节数据
 * @param： cmd：command code
 * @return: 两字节数据
 *************************************************************************/
uint16_t CMelodyLiBattery::read(uint8_t cmd)
{
    uint8_t data[2] = {0};
    read(cmd,data,2);
    return ((uint16_t)data[1] << 8) | data[0];
}

/************************************************************************
 * @brief:  读取数据
 * @param： cmd：command code
 *          pdata,指向读取数据存放的指针
 *          length，读取数据字节数
 * @return：读取的数据
 ************************************************************************/
uint8_t CMelodyLiBattery::read(uint8_t cmd,uint8_t *pdata,uint8_t length)
{
    uint8_t readlength = 0;
    Wire.beginTransmission(BQADDRESS);
    Wire.write(cmd);
    Wire.endTransmission();

    Wire.requestFrom((uint8_t)BQADDRESS,length);
    while(Wire.available())
    {
        pdata[readlength++] = Wire.read();
    }
    delay(1);
    return readlength;
}

//----------public methods----------------------------------//
/***********************************************************************
 * @brief:  初始化配置
 * @param： none
 * @return：none
 ***********************************************************************/
void CMelodyLiBattery::begin(void)
{
    Wire.begin();
}

/**********************************************************************
 * @brief:  获取温度值。可以选择内部温度和外部温度。由[TEMPS] bit in Pack Configuration
 * @param： none
 * @return: 温度值
 **********************************************************************/
float CMelodyLiBattery::getTemperature(void)
{
    uint16_t temperature =  read(BQSTDCMD_TEMPERATURE_LSB);
    //读取的数据单位;0.1K
    return (temperature * 0.1 - 273.1);
}

/************************************************************************
 * @brief:  获取内部温度值
 * @param：  none
 * @return: 温度值
 **************************************************************************/
float CMelodyLiBattery::getInternalTemperature(void)
{
    uint16_t temperature =  read(BQSTDCMD_INTERNALTEMPERATURE_LSB);

    //读取的数据单位;0.1K
    return (temperature * 0.1 - 273.1);
}

/**************************************************************************
 * @brief:  获取电压值
 * @param：  none
 * @return：电压值
 **************************************************************************/
uint16_t CMelodyLiBattery::getVoltage(void)
{
    return read(BQSTDCMD_VOLTAGE_LSB);
}

/**************************************************************************
 * @brief:  获取电流值
 * @param：  none
 * @return：电流值
 **************************************************************************/
int16_t CMelodyLiBattery::getAverageCurrent(void)
{
    return read(BQSTDCMD_AVERAGECURRENT_LSB);
}

/*****************************************************************************
 * @brief:  获取剩余电量
 * @param：  none
 * @return：剩余电量
 *****************************************************************************/
uint16_t CMelodyLiBattery::getRemainingCapacity(void)
{
    return read(BQSTDCMD_REMAININGCAPACITY_LSB);
}

/******************************************************************************
 * @brief:  获取电量百分比
 * @param： none
 * @return：百分比
 ******************************************************************************/
uint8_t CMelodyLiBattery::getStateOfCharge(void)
{
    return read(BQSTDCMD_STATEHARGE_LSB);
}

CMelodyLiBattery MelodyLiBattery;